class Hermes::Api::Dmo::NotificationCenter::SubscriptionsController < Hermes::Api::Dmo::ApplicationController
  include ::NotificationsApiCommonController
  before_action :set_subscription, only: %i[show subscribe unsubscribe]
  before_action :set_params, only: %i[subscribe unsubscribe]

  def index
    paginate(
      ::NotificationCenter::Subscription.where(subscriber_id: @targetable.id),
      each_serializer: serializer,
    )
  end

  def show
    render(jsonapi: @subscription, serializer: serializer)
  end

  def subscribe
    update_subscription('subscribe')
  end

  def unsubscribe
    update_subscription('unsubscribe')
  end

  protected

  def serializer
    ::Hermes::NotificationCenter::SubscriptionSerializer
  end

  def update_subscription(action_type)
    case @provider
    when :email
      @targetable.subscription.send("#{action_type}_to_email", @key)
    when :slack
      @targetable.subscription.send("#{action_type}_to_slack", @key)
    else
      return render_json_error(:not_found, :object_not_found)
    end

    @subscription.reload
    render(jsonapi: @subscription, serializer: serializer)
  end
end