require 'rails_helper'

RSpec.describe Hermes::HousingRequestInitializer, type: :model do
  context 'for structural tests' do
    
    it 'has a valid factory' do
      expect(FactoryBot.build(:housing_request_initializer, unique_number: "1110121021518G1104")).to be_valid
    end
  end
  
  context 'with basic objects' do

    let!(:hri) {create(:housing_request_initializer, unique_number: "1110121021518G1104")}
    
    
    context 'pin code functionality' do
      
      #config.action_mailer.delivery_method = :test
      describe 'pin code generation and email sending' do
        before :each do
          hri.send_pin_code_on_mail
        end
        it 'has save the pin code' do
          hri.reload
          expect(hri.hashed_pin_code).not_to be_empty
          mail_jobs = enqueued_jobs.select {|job| job[:job] == ActionMailer::DeliveryJob}
         # expect(mail_jobs.length).to be >= 1
          mail = perform_enqueued_jobs {ActionMailer::DeliveryJob.perform_now(*mail_jobs.first[:args])}
          expect(mail).not_to eq(nil)
        end
      end
      describe 'pin code verification' do
        it 'validate pin code' do
          pin = hri.send_pin_code_on_mail
          hri.reload
          expect(hri.check_pin_code(pin)).to eq(true)
        end
      end
    end
  end

  describe 'import sne updated renewal booleans' do 
    let!(:user) {create(:user)}
    let!(:applicant) {create(:applicant)}
    let!(:hr) {create(:housing_request, main_applicant: applicant)}

    it 'works' do 
      Timecop.freeze(Time.local(2021, 8, 27, 0, 0))

      xml = Hermes::Xml.create(xml: '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><ns1:interfaceNuu xmlns:ns1="http://nuu.application.i2/" versionInterface="04.00"><ns1:entete><ns1:typeFichier>DIS</ns1:typeFichier><ns1:dateFichier>2021-08-27T14:53:49</ns1:dateFichier></ns1:entete><ns1:demande><ns1:informationDalo><ns1:statutDalo/><ns1:typologieLogementDalo/></ns1:informationDalo><ns1:demandeLogement><ns1:numGuichet>44F01</ns1:numGuichet><ns1:numUnique>044091900020299999</ns1:numUnique><ns1:refInterne>000019692</ns1:refInterne><ns1:dateCreationDemande>2019-09-03</ns1:dateCreationDemande><ns1:forcerDoublon>false</ns1:forcerDoublon><ns1:renouvellementElec>false</ns1:renouvellementElec><ns1:precision></ns1:precision><ns1:ressourcesLoiEC>8472</ns1:ressourcesLoiEC><ns1:anru>false</ns1:anru><ns1:ressourcesMensuelles>true</ns1:ressourcesMensuelles><ns1:logementRecherche><ns1:categorieLogementRecherche code="I"/><ns1:avecParking>true</ns1:avecParking><ns1:refusRezDeChaussee>false</ns1:refusRezDeChaussee><ns1:refusSansAscenseur>true</ns1:refusSansAscenseur><ns1:demandeElargie>true</ns1:demandeElargie><ns1:infoProprietaire>false</ns1:infoProprietaire><ns1:listeNombrePieces><ns1:nombrePiecesRecherchees code="P3"/><ns1:nombrePiecesRecherchees code="P4"/></ns1:listeNombrePieces><ns1:listeLocalisationSouhaite><ns1:localisationSouhaite><ns1:commune code="44035"/><ns1:codePostal>44240</ns1:codePostal><ns1:seuilEPCIPremierQuartile>8800</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>1</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="44215"/><ns1:codePostal>44120</ns1:codePostal><ns1:seuilEPCIPremierQuartile>8800</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>1</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="44198"/><ns1:codePostal>44840</ns1:codePostal><ns1:seuilEPCIPremierQuartile>8800</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>1</ns1:quartileAppartenance></ns1:localisationSouhaite><ns1:localisationSouhaite><ns1:commune code="44172"/><ns1:codePostal>44980</ns1:codePostal><ns1:seuilEPCIPremierQuartile>8800</ns1:seuilEPCIPremierQuartile><ns1:quartileAppartenance>1</ns1:quartileAppartenance></ns1:localisationSouhaite></ns1:listeLocalisationSouhaite></ns1:logementRecherche><ns1:personnePhysique><ns1:demandeur><ns1:civilite code="2"/><ns1:nomPatronymique>HUOUME</ns1:nomPatronymique><ns1:prenoms>Martine</ns1:prenoms><ns1:dateNaissance>1994-03-13</ns1:dateNaissance><ns1:nationalite code="1"/><ns1:mel>marieclinchardals+10@gmail.com</ns1:mel><ns1:notifSms>false</ns1:notifSms><ns1:adressePostale><ns1:etranger>false</ns1:etranger><ns1:appartement>28</ns1:appartement><ns1:etage>9</ns1:etage><ns1:batiment>3</ns1:batiment><ns1:numero>3</ns1:numero><ns1:voie>RUE PROSPER MERIMEE</ns1:voie><ns1:lieuDit/><ns1:codePostal>44300</ns1:codePostal><ns1:commune code="44109"/></ns1:adressePostale><ns1:situationProfessionnelle><ns1:typeContratTravail code="CHO"/></ns1:situationProfessionnelle><ns1:revenuFiscal><ns1:anneeMoins1>2019</ns1:anneeMoins1><ns1:montantMoins1>0</ns1:montantMoins1><ns1:anneeMoins2>2018</ns1:anneeMoins2><ns1:montantMoins2>0</ns1:montantMoins2></ns1:revenuFiscal><ns1:listeRessourceRecue><ns1:detailRessource><ns1:ressource code="AUTR"/><ns1:montant>193</ns1:montant></ns1:detailRessource><ns1:detailRessource><ns1:ressource code="PDA"/><ns1:montant>156</ns1:montant></ns1:detailRessource><ns1:detailRessource><ns1:ressource code="RSA"/><ns1:montant>568</ns1:montant></ns1:detailRessource></ns1:listeRessourceRecue><ns1:listeRessourceVersee><ns1:detailRessource><ns1:ressource code="PAVS"/><ns1:montant>0</ns1:montant></ns1:detailRessource></ns1:listeRessourceVersee><ns1:listeRessourceRecuePersACharge/><ns1:listeRessourceVerseePersACharge/></ns1:demandeur><ns1:adresseLogement><ns1:etranger>false</ns1:etranger><ns1:appartement>28</ns1:appartement><ns1:etage>9</ns1:etage><ns1:batiment>3</ns1:batiment><ns1:numero>3</ns1:numero><ns1:voie>RUE PROSPER MERIMEE</ns1:voie><ns1:lieuDit/><ns1:codePostal>44300</ns1:codePostal><ns1:commune code="44109"/></ns1:adresseLogement><ns1:listePersonneACharge><ns1:personneACharge><ns1:nom>PREOP</ns1:nom><ns1:prenom>Marie</ns1:prenom><ns1:dateNaissance>2015-11-23</ns1:dateNaissance><ns1:sexe code="F"/><ns1:lienParente code="E"/></ns1:personneACharge></ns1:listePersonneACharge><ns1:listeCodemandeur/><ns1:situation><ns1:listeModeleLogement><ns1:modeleLogement code="LLS"/></ns1:listeModeleLogement><ns1:listeSituation><ns1:detailSituation code="TCH"/><ns1:detailSituation code="TTP"/><ns1:detailSituation code="LIN"/></ns1:listeSituation><ns1:compNomOrgBailleur>Nantes Métropole Habitat</ns1:compNomOrgBailleur><ns1:compSIRENOrgBailleur>274400027</ns1:compSIRENOrgBailleur><ns1:montantMensuelLoyer>375</ns1:montantMensuelLoyer><ns1:montantMensuelAPL>247</ns1:montantMensuelAPL><ns1:nombreHabitant>2</ns1:nombreHabitant><ns1:nombrePiecesActuel code="P3"/><ns1:surface>55</ns1:surface><ns1:proprietaire>false</ns1:proprietaire></ns1:situation></ns1:personnePhysique><ns1:informationHandicap><ns1:nombreHandicap>0</ns1:nombreHandicap></ns1:informationHandicap><ns1:renouvellement><ns1:dateRenouvellement>2020-09-20</ns1:dateRenouvellement></ns1:renouvellement></ns1:demandeLogement></ns1:demande></ns1:interfaceNuu>')
      hri = Hermes::HousingRequestInitializer.create(unique_number: '044091900020299999', xml: xml)
      hri.hydrate_housing_request_from_sne(user.id) 

      expect(hri.housing_request.reload.soon_need_to_be_renewed).to eq(true)
      expect(hri.housing_request.reload.need_to_be_renewed).to eq(false)
      expect(hri.housing_request.reload.next_sne_renewal_date).to eq(Date.new(2021,9,20))

      Timecop.return
    end 
  end 
end
